@@ -14,6 +14,12 @@ import astropy.units as u
 from . import db
 from .. import utils
 
+ROOT_PATH = '/GrizliImaging'
+if not os.path.exists(ROOT_PATH):
+    ROOT_PATH = os.getcwd()
+    print(f'Set ROOT_PATH={ROOT_PATH}')
+
+
 def s3_object_path(dataset, product='raw', ext='fits', base_path='hst/public/'):
     """
     S3 path for an HST ``dataset``
@@ -485,7 +491,11 @@ def add_wcs_log(files=None, assoc=None, remove_old=True, verbose=True):
             
         parent = os.path.basename(file).split('_wcs.log')[0]
         if remove_old:
-            db.execute(f"DELETE FROM wcs_log WHERE wcs_parent='{parent}'")
+            cmd = f"DELETE FROM wcs_log WHERE wcs_parent='{parent}'"
+            if assoc is not None:
+                cmd += f" AND wcs_assoc='{assoc}'"
+                
+            db.execute(cmd)
             
         rows = []
         
@@ -543,7 +553,7 @@ def update_assoc_status(assoc, status=1, verbose=True):
     
     sqlstr = """UPDATE {0}
         SET status = {1}, modtime = '{2}'
-        WHERE (assoc_name = '{3}');
+        WHERE (assoc_name = '{3}' AND status != 99);
         """.format(table, status, NOW, assoc)
 
     if verbose:
@@ -596,8 +606,8 @@ def delete_all_assoc_data(assoc):
     os.system(f'aws s3 rm --recursive s3://grizli-v2/HST/Pipeline/{assoc}')
 
     res = db.execute(f"""UPDATE assoc_table
-              SET status=12
-                    WHERE assoc_name = '{assoc}'""")
+                    SET status=12
+                    WHERE assoc_name = '{assoc}' AND status != 99""")
 
 
 def clear_failed():
@@ -674,14 +684,14 @@ def show_recent_assoc(limit=50):
     return last
 
 
-def launch_ec2_instances(nmax=50, count=None, templ='lt-0e8c2b8611c9029eb,Version=19'):
+def launch_ec2_instances(nmax=50, count=None, templ='lt-0e8c2b8611c9029eb,Version=24'):
     """
     Launch EC2 instances from a launch template that run through all 
     status=0 associations/tiles and then terminate
     
     Version 19 is the latest run_all_visits.sh
     Version 20 is the latest run_all_tiles.sh
-    
+    Version 24 is run_all_visits with a new python39 environment
     """
 
     if count is None:
@@ -725,7 +735,7 @@ def get_assoc_yaml_from_s3(assoc, s_region=None, bucket='grizli-v2', prefix='HST
     from grizli.pipeline import auto_script
     from grizli import utils
     
-    LOGFILE = f'/GrizliImaging/{assoc}.auto_script.log.txt'
+    LOGFILE = f'{ROOT_PATH}/{assoc}.auto_script.log.txt'
     
     s3 = boto3.resource('s3')
     s3_client = boto3.client('s3')
@@ -765,7 +775,7 @@ def get_assoc_yaml_from_s3(assoc, s_region=None, bucket='grizli-v2', prefix='HST
             files = [obj.key for obj in
                      file_bkt.objects.filter(Prefix=file_prefix)]
             if len(files) > 0:
-                local_file = os.path.join('/GrizliImaging/', 
+                local_file = os.path.join(f'{ROOT_PATH}/', 
                                           os.path.basename(file_prefix))
                 
                 if not os.path.exists(local_file):
@@ -1076,10 +1086,111 @@ blue_align_params['align_min_flux_radius'] = 1.7
 blue_align_params['tweak_n_min'] = 5
 
 
+def check_jwst_assoc_guiding(assoc):
+    """
+    Check the guidestar logs that accompany a given visit
+    """
+    import os
+    import glob
+    
+    import matplotlib.pyplot as plt
+    
+    import astropy.table
+    
+    import mastquery.utils
+    import mastquery.jwst
+    
+    os.chdir(f'{ROOT_PATH}/')
+    
+    atab = db.SQL(f"""SELECT t_min, t_max, filter, proposal_id, 
+                             "dataURL", status
+                     FROM assoc_table
+                     WHERE assoc_name='{assoc}'
+                     """)
+    so = np.argsort(atab['t_min'])
+    atab = atab[so]
+    
+    gs = mastquery.jwst.query_guidestar_log(
+             mjd=(atab['t_min'].min()-0.1, atab['t_max'].max()+0.1),
+             program=atab['proposal_id'][0],
+             exp_type=['FGS_FINEGUIDE'],
+         )
+    
+    keep = (gs['expstart'] < atab['t_max'].max())
+    gs = gs[keep]
+    
+    for _iter in range(3):
+        res = mastquery.utils.download_from_mast(gs)
+    
+    tabs = []
+    for f in res:
+        if not os.path.exists(f):
+            continue
+            
+        tabs.append(utils.read_catalog(f))
+    
+    tab = astropy.table.vstack(tabs)
+    so = np.argsort(tab['time'])
+    tab = tab[so]
+    
+    fig, ax = plt.subplots(1,1,figsize=(8,5))
+    t0 = atab['t_min'].min()
+    ax.scatter((tab['time'] - t0)*24, tab['jitter'],
+               alpha=.02, marker='.', color='k')
+    
+    atab['jitter16'] = -1.
+    atab['jitter50'] = -1.
+    atab['jitter84'] = -1.
+    
+    ymax = 500
+    
+    for i, row in enumerate(atab):
+        gsx = (tab['time'] > row['t_min']) & (tab['time'] < row['t_max'])
+        
+        if gsx.sum() == 0:
+            gs_stats = [-1,-1,-1]
+        else:
+            gs_stats = np.percentile(tab['jitter'][gsx], [16, 50, 84])
+
+        atab['jitter16'][i] = gs_stats[0]
+        atab['jitter50'][i] = gs_stats[1]
+        atab['jitter84'][i] = gs_stats[2]
+        
+        print(f"{i} {os.path.basename(atab['dataURL'][i])} {gs_stats[1]:.2f}")
+        
+        ax.fill_between([(row['t_min']-t0)*24, (row['t_max']-t0)*24], 
+                        np.ones(2)*gs_stats[0], np.ones(2)*gs_stats[2], 
+                        color='r', alpha=0.2)
+        ax.hlines(gs_stats[1], (row['t_min']-t0)*24, (row['t_max']-t0)*24, 
+                  color='r')
+        ax.fill_between([(row['t_min']-t0)*24, (row['t_max']-t0)*24], 
+                        [0.2,0.2], np.ones(2)*(0.12/0.2*ymax), 
+                        color='0.8', alpha=0.1, zorder=-1)
+                        
+    ax.set_ylim(0.12, ymax)
+    ax.semilogy()
+    
+    ax.set_xlabel(r'$\Delta t$, hours since ' + f'{t0:.2f}')
+    ax.set_ylabel('GuideStar jitter, mas')
+    ax.set_yticklabels([1,10,100])
+    ax.set_yticks([1,10,100])
+    ax.set_title(assoc)
+    
+    ax.set_xlim(-0.2, (atab['t_max'].max() - t0)*24 +0.2)
+    ax.grid()
+    
+    fig.tight_layout(pad=0.5)
+    fig.savefig(f'{assoc}.guide.png')
+    
+    atab.write(f'{assoc}.guide.csv', overwrite=True)
+    
+    return fig, atab
+    
+
 ALL_FILTERS = ['F410M', 'F467M', 'F547M', 'F550M', 'F621M', 'F689M', 'F763M', 'F845M', 'F200LP', 'F350LP', 'F435W', 'F438W', 'F439W', 'F450W', 'F475W', 'F475X', 'F555W', 'F569W', 'F600LP', 'F606W', 'F622W', 'F625W', 'F675W', 'F702W', 'F775W', 'F791W', 'F814W', 'F850LP', 'G800L', 'F098M', 'F127M', 'F139M', 'F153M', 'F105W', 'F110W', 'F125W', 'F140W', 'F160W', 'G102', 'G141']
 
 
-def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False, visit_split_shift=1.2, blue_align_params=blue_align_params, ref_catalogs=['LS_DR9', 'PS1', 'DES', 'NSC', 'GAIA'], filters=None, prep_args={}, get_wcs_guess_from_table=True, master_radec='astrometry_db', **kwargs):
+def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False, visit_split_shift=1.2, blue_align_params=blue_align_params, ref_catalogs=['LS_DR9', 'PS1', 'DES', 'NSC', 'GAIA'], filters=None, prep_args={}, fetch_args={}, get_wcs_guess_from_table=True, master_radec='astrometry_db', align_guess=None, **kwargs):
     """
     `assoc_table.status`
     
@@ -1099,10 +1210,12 @@ def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False,
     from grizli.pipeline import auto_script
     from grizli import utils, prep
 
-    os.chdir('/GrizliImaging/')
+    os.chdir(f'{ROOT_PATH}/')
     
     tab = db.SQL(f"""SELECT * FROM assoc_table
-                     WHERE assoc_name='{assoc}'""")
+                     WHERE assoc_name='{assoc}'
+                     AND status != 99
+                     """)
     
     if len(tab) == 0:
         print(f"assoc_name='{assoc}' not found in assoc_table")
@@ -1120,7 +1233,7 @@ def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False,
     
     if get_wcs_guess_from_table:
         get_wcs_guess(assoc)
-      
+    
     keep = []
     for c in tab.colnames:
         if tab[c].dtype not in [object]:
@@ -1155,7 +1268,10 @@ def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False,
     
     for k in prep_args:
         kws['visit_prep_args'][k] = prep_args[k]
-                
+    
+    for k in fetch_args:
+        kws['fetch_files_args'][k] = fetch_args[k]
+        
     kws['kill'] = 'preprocess'
     
     if master_radec is not None:
@@ -1211,7 +1327,7 @@ def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False,
     os.environ['iref'] = os.environ['orig_iref']
     os.environ['jref'] = os.environ['orig_jref']
     
-    os.chdir('/GrizliImaging/')
+    os.chdir(f'{ROOT_PATH}/')
     
     if sync:
         os.system(f'aws s3 rm --recursive ' + 
@@ -1232,7 +1348,7 @@ def process_visit(assoc, clean=True, sync=True, max_dt=4, combine_same_pa=False,
         
             os.system(cmd)
             print('\n# Sync\n' + cmd.replace('     ', '') + '\n')
-            os.chdir('/GrizliImaging')
+            os.chdir(f'{ROOT_PATH}')
             
         files = glob.glob(f'{assoc}*.*')
         for file in files:
@@ -1264,10 +1380,19 @@ def set_private_iref(assoc):
     os.environ['iref'] = f'{os.getcwd()}/{assoc}/iref/'
     os.environ['jref'] = f'{os.getcwd()}/{assoc}/jref/'
     
-    utils.fetch_default_calibs()
+    is_jwst = False
+    for f in ['f090w','f115w','f150w','f200w','f277w','f356w','f444w',
+              'f182m','f140m','f210m','f410m','f430m','f460m','f300m',
+              'f250m','f480m']:
+        if f in assoc:
+            is_jwst = True
+            break
+    
+    if not is_jwst:        
+        utils.fetch_default_calibs()
 
 
-def get_wcs_guess(assoc, verbose=True):
+def get_wcs_guess(assoc, guess=None, verbose=True):
     """
     Get visit wcs alignment from the database and use it as a "guess"
     """
@@ -1275,7 +1400,7 @@ def get_wcs_guess(assoc, verbose=True):
     
     wcs = db.from_sql(f"select * from wcs_log where wcs_assoc = '{assoc}'")
     
-    LOGFILE = f'/GrizliImaging/{assoc}.auto_script.log.txt'
+    LOGFILE = f'{ROOT_PATH}/{assoc}.auto_script.log.txt'
     
     if len(wcs) == 0:
         msg = f"# get_wcs_guess : No entries found in wcs_log for wcs_assoc='{assoc}'"
@@ -1302,7 +1427,7 @@ def get_wcs_guess(assoc, verbose=True):
             fp.write(guess)        
         
             
-def make_parent_mosaic(parent='j191436m5928', **kwargs):
+def make_parent_mosaic(parent='j191436m5928', root=None, **kwargs):
     """
     Get the full footprint of all exposure in the database for a given 'parent'
     """
@@ -1329,7 +1454,10 @@ def make_parent_mosaic(parent='j191436m5928', **kwargs):
     size = ((np.max(bx) - np.min(bx))*cosd*3600+10., 
             (np.max(by) - np.min(by))*3600+10.)
     
-    cutout_mosaic(rootname=parent, ra=ra, dec=dec, size=size, **kwargs)
+    if root is None:
+        root = parent
+        
+    cutout_mosaic(rootname=root, ra=ra, dec=dec, size=size, **kwargs)
 
 
 def res_query_from_local(files=None, filters=None):
@@ -1401,11 +1529,132 @@ def res_query_from_local(files=None, filters=None):
     return res
 
 
-def cutout_mosaic(rootname='gds', product='{rootname}-{f}', ra=53.1615666, dec=-27.7910651, size=5*60, filters=['F160W'], ir_scale=0.1, ir_wcs=None, res=None, half_optical=True, kernel='point', pixfrac=0.33, make_figure=True, skip_existing=True, clean_flt=True, gzip_output=True, s3output='s3://grizli-v2/HST/Pipeline/Mosaic/', split_uvis=True, extra_query='', extra_wfc3ir_badpix=True, fix_niriss=True, scale_nanojy=10, verbose=True, niriss_ghost_kwargs={}, **kwargs):
+def query_exposures(ra=53.16, dec=-27.79, size=1., pixel_scale=0.1, theta=0,  filters=['F160W'], wcs=None, extra_columns=[], extra_query='', res=None, max_wcs=False):
+    """
+    Query exposures that overlap a WCS, either specified explicitly or 
+    generated from position parameters
+    
+    Parameters
+    ----------
+    ra : float
+        RA center, decimal degrees
+    
+    dec : float
+        Dec center, decimal degrees
+    
+    size : float
+        Point query, arcsec
+    
+    pixel_scale : float
+        Pixel scale, arcsec
+    
+    theta : float
+        WCS position angle
+        
+    filters : list, None
+        Filter list
+    
+    wcs : `~astropy.wcs.WCS`, None
+        WCS to supersede ``ra``, ``dec``, ``size``, ``pixel_scale``
+    
+    extra_columns : string
+        Additional columns to query, start with a ','
+        
+    extra_query : str
+        Extra criteria appended to SQL query
+    
+    Returns
+    -------
+    header : `~astropy.io.fits.Header`
+        WCS header
+        
+    wcs : `~astropy.wcs.WCS`
+        Query WCS
+        
+    SQL : str
+        Query string
+        
+    res : `~grizli.utils.GTable`
+        Query result
+    
+    """
+    import astropy.wcs as pywcs
+    
+    if wcs is None:
+        _hdu = utils.make_wcsheader(ra=ra, dec=dec, size=size,
+                                    pixscale=pixel_scale,
+                                    theta=theta,
+                                    get_hdu=True)
+        header = _hdu.header
+        wcs = pywcs.WCS(header)
+    else:
+        header = utils.to_header(wcs)
+        
+    fp = wcs.calc_footprint()
+        
+    x1, y1 = fp.min(axis=0)
+    x2, y2 = fp.max(axis=0)
+    
+    # Database query
+    SQL = f"""
+    SELECT dataset, extension, sciext, assoc, 
+           e.filter, e.pupil, e.exptime, e.footprint, e.detector
+           {extra_columns}
+    FROM exposure_files e, assoc_table a
+    WHERE e.assoc = a.assoc_name
+    AND a.status = 2
+    AND e.exptime > 0
+    AND polygon(e.footprint) && polygon(box '(({x1},{y1}),({x2},{y2}))') 
+    {extra_query}   
+    """
+    
+    if filters is not None:
+        filter_sql = ' OR '.join([f"e.filter = '{f}'" for f in filters])
+        SQL += f'AND ({filter_sql})'
+    
+    SQL += ' ORDER BY e.filter'
+    if res is None:
+        res = db.SQL(SQL)
+    
+    if max_wcs:
+        fp = None
+        for f in res['footprint']:
+            fp_i = utils.SRegion(f)
+
+            for fs in fp_i.shapely:
+                if fp is None:
+                    fp = fs
+                else:
+                    fp = fp.union(fs)
+
+        bx, by = fp.convex_hull.boundary.xy
+        ra = (np.max(bx) + np.min(bx))/2
+        dec = (np.max(by) + np.min(by))/2
+
+        cosd = np.cos(dec/180*np.pi)
+
+        size = (np.max(np.abs(bx-ra))*2*cosd*3600 + max_wcs*1., 
+                np.max(np.abs(by-dec))*2*3600 + max_wcs*1.)
+        
+        _hdu = utils.make_wcsheader(ra=ra, dec=dec, size=size,
+                                    pixscale=pixel_scale,
+                                    theta=theta,
+                                    get_hdu=True)
+        header = _hdu.header
+        wcs = pywcs.WCS(header)
+        
+    return header, wcs, SQL, res
+
+
+def cutout_mosaic(rootname='gds', product='{rootname}-{f}', ra=53.1615666, dec=-27.7910651, size=5*60, theta=0., filters=['F160W'], ir_scale=0.1, ir_wcs=None, res=None, half_optical=True, kernel='point', pixfrac=0.33, make_figure=True, skip_existing=True, clean_flt=True, gzip_output=True, s3output='s3://grizli-v2/HST/Pipeline/Mosaic/', split_uvis=True, extra_query='', extra_wfc3ir_badpix=True, fix_niriss=True, scale_nanojy=10, verbose=True, niriss_ghost_kwargs={}, scale_photom=True, **kwargs):
     """
     Make mosaic from exposures defined in the exposure database
     
-        
+    Parameters
+    ----------
+    
+    Returns
+    -------
     """
     import os
     import glob
@@ -1419,8 +1668,15 @@ def cutout_mosaic(rootname='gds', product='{rootname}-{f}', ra=53.1615666, dec=-
     
     utils.set_warnings()
     
+    # out_h, ir_wcs, res = query_exposures(ra=ra, dec=dec,
+    #                                      size=size,
+    #                                      pixel_scale=pixel_scale,
+    #                                      theta=theta,
+    #                                      filters=filters,
+    #                                      wcs=ir_wcs,
+    #                                      extra_query=extra_query)
     if ir_wcs is None:
-        out_h, ir_wcs = utils.make_wcsheader(ra=ra, dec=dec, size=size, 
+        out_h, ir_wcs = utils.make_wcsheader(ra=ra, dec=dec, size=size,
                                           pixscale=ir_scale, get_hdu=False)
     
     fp = ir_wcs.calc_footprint()
@@ -1428,8 +1684,8 @@ def cutout_mosaic(rootname='gds', product='{rootname}-{f}', ra=53.1615666, dec=-
     if half_optical:
         opt_wcs = utils.half_pixel_scale(ir_wcs)
         
-    x1, y1 = fp[0]
-    x2, y2 = fp[2]
+    x1, y1 = fp.min(axis=0)
+    x2, y2 = fp.max(axis=0)
     
     # Database query
     SQL = f"""
@@ -1567,6 +1823,7 @@ def cutout_mosaic(rootname='gds', product='{rootname}-{f}', ra=53.1615666, dec=-
                                      kernel=kernel, clean=clean_flt, 
                                      extra_wfc3ir_badpix=extra_wfc3ir_badpix,
                                      verbose=verbose,
+                                     scale_photom=scale_photom,
                                      niriss_ghost_kwargs=niriss_ghost_kwargs)
                              
         outsci, outwht, header, flist, wcs_tab = _
@@ -1618,6 +1875,7 @@ def cutout_mosaic(rootname='gds', product='{rootname}-{f}', ra=53.1615666, dec=-
 
             files += glob.glob(f'{prod}*fits*')
             files += glob.glob(f'{prod}*_fp.png')
+            files += glob.glob(f'{prod}*wcs.csv')
         
         
         for file in files:
@@ -1879,14 +2137,14 @@ def run_one(clean=2, sync=True):
     
     assoc = get_random_visit()
     if assoc is None:
-        with open('/GrizliImaging/finished.txt','w') as fp:
+        with open(f'{ROOT_PATH}/finished.txt','w') as fp:
             fp.write(time.ctime() + '\n')
     else:
         print(f'============  Run association  ==============')
         print(f'{assoc}')
         print(f'========= {time.ctime()} ==========')
         
-        with open('/GrizliImaging/visit_history.txt','a') as fp:
+        with open(f'{ROOT_PATH}/visit_history.txt','a') as fp:
             fp.write(f'{time.ctime()} {assoc}\n')
         
         process_visit(assoc, clean=clean, sync=sync)
