@@ -462,7 +462,38 @@ class aXeConf():
         #plt.savefig('{0}.pdf'.format(self.conf_file))
         
         return fig
-        
+
+
+def coeffs_from_astropy_polynomial(p):
+    """
+    Get field-dependent coefficients in aXe format from an 
+    `astropy.modeling.polynomial.Polynomial2D` model
+    
+    Parameters
+    ----------
+    p : `astropy.modeling.polynomial.Polynomial2D`
+        Polynomial model
+    
+    Returns
+    -------
+    coeffs : array-like
+        Reordered array of coefficients
+    
+    """
+    coeffs = []
+    for _p in range(p.degree+1):
+        for _py in range(_p+1):
+            # print 'x**%d y**%d' %(_p-_py, _py)
+            _px = _p - _py
+            pname = f'c{_px}_{_py}'
+            if pname in p.param_names:
+                pix = p.param_names.index(pname)
+                coeffs.append(p.parameters[pix])
+            else:
+                coeffs.append(0.0)
+    
+    return np.array(coeffs)
+
 
 def get_config_filename(instrume='WFC3', filter='F140W',
                         grism='G141', module=None, chip=1):
@@ -534,6 +565,8 @@ def get_config_filename(instrume='WFC3', filter='F140W',
     if instrume == 'NIRISS':
         
         conf_files = []
+        conf_files.append(os.path.join(GRIZLI_PATH,
+                            'CONF/{0}.{1}.221215.conf'.format(grism, filter)))
         conf_files.append(os.path.join(GRIZLI_PATH,
                             'CONF/{0}.{1}.220725.conf'.format(grism, filter)))
         conf_files.append(os.path.join(GRIZLI_PATH,
@@ -545,6 +578,9 @@ def get_config_filename(instrume='WFC3', filter='F140W',
             if os.path.exists(conf_file):
                 #print(f'NIRISS: {conf_file}')
                 break
+            else:
+                #print(f'skip NIRISS: {conf_file}')
+                pass
                 
         # if not os.path.exists(conf_file):
         #     print('CONF/{0}.{1}.conf'.format(grism, filter))
@@ -560,11 +596,14 @@ def get_config_filename(instrume='WFC3', filter='F140W',
         
         fi = grism
         gr = filter[-1] # R, C
-        conf_file = os.path.join(GRIZLI_PATH,
-                    f'CONF/GRISM_NIRCAM/gNIRCAM.{fi}.mod{module}.{gr}.conf')
+        # conf_file = os.path.join(GRIZLI_PATH,
+        #             f'CONF/GRISM_NIRCAM/gNIRCAM.{fi}.mod{module}.{gr}.conf')
         #
+        # conf_file = os.path.join(GRIZLI_PATH,
+        #             f'CONF/GRISM_NIRCAM/V2/NIRCAM_{fi}_mod{module}_{gr}.conf')
+
         conf_file = os.path.join(GRIZLI_PATH,
-                    f'CONF/GRISM_NIRCAM/V2/NIRCAM_{fi}_mod{module}_{gr}.conf')
+                    f'CONF/GRISM_NIRCAM/V4/NIRCAM_{fi}_mod{module}_{gr}.conf')
         
     elif instrume == 'NIRCAMA':
         fi = grism
@@ -865,6 +904,8 @@ class TransformGrismconf(object):
             Effective wavelength along the trace evaluated at `dx`.
 
         """
+        from astropy.modeling.models import Polynomial2D
+        
         x0 = np.squeeze(self.transform.reverse(x, y))
         
         if self.transform.trace_axis == '+x':
@@ -894,7 +935,45 @@ class TransformGrismconf(object):
         rev = self.transform.forward(x0[0]+tdx, x0[1]+tdy)
         trace_dy = rev[1,:] - y
         #trace_dy = y - rev[1,:]
+        
+        # Trace offsets for NIRCam
+        if 'V4/NIRCAM_F444W_modB_R.conf' in self.conf_file:
+            trace_dy += -0.5
+            
+            # Shifts derived from FRESCO
+            coeffs = {'c0_0': 0.3723992993620532,
+                      'c1_0': -0.00011461411413576305,
+                      'c2_0': -8.575199405062535e-08,
+                      'c0_1': -0.0011862122093603026,
+                      'c0_2': 4.1403439215806165e-07,
+                      'c1_1': 1.6558275336712723e-07
+                     }
+            
+            poly = Polynomial2D(degree=2, **coeffs)
+            trace_dy += poly(x, y)
+            #print(f'polynomial offset: {poly(x,y):.3f}')
+            
+        elif 'V4/NIRCAM_F444W_modA_R.conf' in self.conf_file:
+            trace_dy += -2.5
+            
+            # Shifts derived from FRESCO
+            coeffs = {'c0_0': 0.34191256988768415,
+                      'c1_0': -0.0003378232293429956,
+                      'c2_0': -9.238111910134196e-09,
+                      'c0_1': -7.063720696711682e-05,
+                      'c0_2': 2.5217177632321527e-08,
+                      'c1_1': -1.4345820074275903e-07
+                      }
 
+            poly = Polynomial2D(degree=2, **coeffs)
+            trace_dy += poly(x, y)
+            #print(f'polynomial offset: {poly(x,y):.3f}')
+            
+        elif os.path.basename(self.conf_file) == 'NIRCAM_F444W_modA_R.conf':
+            trace_dy += -2.5
+        elif os.path.basename(self.conf_file) == 'NIRCAM_F444W_modA_C.conf':
+            trace_dy += -0.1
+            
         wave = self.conf.DISPL(self.order_names[beam], *x0, t)
         if self.transform.instrument != 'HST':
             wave *= 1.e4
@@ -943,7 +1022,7 @@ class TransformGrismconf(object):
             #self.beams.append(beam)
             self.dxlam[beam] = np.arange(xarr[0], xarr[-1], dtype=int)
             self.nx[beam] = xarr[-1] - xarr[0]+1
-
+                        
             sens = Table()
             sens['WAVELENGTH'] = lam.astype(np.double)
             sens['SENSITIVITY'] = self.conf.SENS[order](lam).astype(np.double)
@@ -954,6 +1033,19 @@ class TransformGrismconf(object):
             
             self.conf_dict[f'BEAM{beam}'] = np.array([xarr[0], xarr[-1]])
             self.conf_dict[f'MMAG_EXTRACT_{beam}'] = 29
+        
+        # Read updated sensitivity files
+        if 'V4/NIRCAM_F444W' in self.conf_file:
+            sens_file = self.conf_file.replace('.conf', '_ext_sensitivity.fits')
+            if os.path.exists(sens_file):
+                # print(f'Replace sensitivity: {sens_file}')
+                new = utils.read_catalog(sens_file)
+                _tab = utils.GTable()
+                _tab['WAVELENGTH'] = new['WAVELENGTH'].astype(float)
+                _tab['SENSITIVITY'] = new['SENSITIVITY'].astype(float)
+                _tab['ERROR'] = new['ERROR'].astype(float)
+                
+                self.sens['A'] = _tab
 
 
 def load_grism_config(conf_file, warnings=True):
@@ -976,28 +1068,34 @@ def load_grism_config(conf_file, warnings=True):
     elif 'V2/NIRCAM' in conf_file:
         conf = TransformGrismconf(conf_file)
         conf.get_beams()
+    elif 'V4/NIRCAM' in conf_file:
+        conf = TransformGrismconf(conf_file)
+        conf.get_beams()
     else:
         conf = aXeConf(conf_file)
         conf.get_beams()
     
     # Preliminary hacks for on-sky NIRISS
     if 'GR150' in conf_file:
-        hack_niriss = 1./1.8 * 1.1
-        
-        msg = f"""
- ! Scale NIRISS sensitivity by {hack_niriss:.3f} to hack gain correction
- ! and match GLASS MIRAGE simulations. Sensitivity will be updated when
- ! on-sky data available
- """
-        msg = f' ! Scale NIRISS sensitivity by {hack_niriss:.3f} prelim flux correction'
-        utils.log_comment(utils.LOGFILE, msg, verbose=warnings)
+        if 0:
+            hack_niriss = 1./1.8 * 1.1
         
+            msg = f"""
+     ! Scale NIRISS sensitivity by {hack_niriss:.3f} to hack gain correction
+     ! and match GLASS MIRAGE simulations. Sensitivity will be updated when
+     ! on-sky data available
+     """
+            msg = f' ! Scale NIRISS sensitivity by {hack_niriss:.3f} prelim flux correction'
+            utils.log_comment(utils.LOGFILE, msg, verbose=warnings)
+        else:
+            hack_niriss = 1.0
+            
         for b in conf.sens:
             conf.sens[b]['SENSITIVITY'] *= hack_niriss
             if 'ERROR' in conf.sens[b].colnames:
                 conf.sens[b]['ERROR'] *= hack_niriss
         
-        if 'F115W' in conf_file:
+        if ('F115W' in conf_file) | ('.2212' in conf_file):
             pass
             # msg = f""" !! Shift F115W along dispersion"""
             # utils.log_comment(utils.LOGFILE, msg, verbose=warnings)
@@ -1024,11 +1122,11 @@ def load_grism_config(conf_file, warnings=True):
         #     # print('xxx', conf_file, _w0)
         #     conf.sens['A']['SENSITIVITY'] *= slope
         
-        if 'F150W' in conf_file:
+        if ('F150W' in conf_file) & (hack_niriss > 1.01):
             conf.sens['A']['SENSITIVITY'] *= 1.08
-            
-        # Scale 0th orders in F150W,F200W
-        if ('F150W' in conf_file) | ('F200W' in conf_file):
+             
+        # Scale 0th orders in F150W
+        if ('F150W' in conf_file): # | ('F200W' in conf_file):
             msg = f""" ! Scale 0th order (B) by an additional x 1.5"""
             utils.log_comment(utils.LOGFILE, msg, verbose=warnings)
             conf.sens['B']['SENSITIVITY'] *= 1.5
