@@ -7,16 +7,19 @@ Requires https://github.com/spacetelescope/jwst
 import os
 import inspect
 import logging
+import traceback
 
 import astropy.io.fits as pyfits
 import astropy.wcs as pywcs
 
 import numpy as np
 from . import utils
+from . import GRIZLI_PATH
 
 QUIET_LEVEL = logging.INFO
 
-CRDS_CONTEXT = 'jwst_0942.pmap' # July 29, 2022 with updated NIRCAM ZPs
+# CRDS_CONTEXT = 'jwst_0942.pmap' # July 29, 2022 with updated NIRCAM ZPs
+CRDS_CONTEXT = 'jwst_0995.pmap' # 2022-10-06 NRC ZPs and flats
 
 def set_crds_context(fits_file=None, override_environ=False, verbose=True):
     """
@@ -159,7 +162,109 @@ def change_header_pointing(header, ra_ref=0., dec_ref=0., pa_v3=0.):
     return new_header
 
 
-def img_with_flat(input, verbose=True, overwrite=True, apply_photom=True):
+def get_jwst_skyflat(header, verbose=True, valid_flat=(0.7, 1.4)):
+    """
+    Get sky flat for JWST instruments
+    
+    Parameters
+    ----------
+    header : `astropy.io.fits.Header`
+        Primary header
+    
+    verbose : bool
+        Verbose messaging
+    
+    valid_flat : (float, float)
+        Range of values to define where the flat is valid to avoid corrections
+        that are too large
+    
+    Returns
+    -------
+    skyfile : str
+        Filename of the sky flat file
+    
+    flat_corr : array-like
+        The flat correction, equal to the original flat divided by the 
+        new sky flat, i.e., to take out the former and apply the latter
+    
+    dq : array-like
+        DQ array with 1024 where flat outside of ``valid_flat`` range
+    
+    If no flat file is found, returns ``None`` for all outputs
+    
+    """
+    filt = utils.parse_filter_from_header(header)
+
+    key = ('{0}-{1}'.format(header['detector'], filt)).lower()
+    conf_path = os.path.join(GRIZLI_PATH, 'CONF', 'NircamSkyFlat')
+    if 'nrcb4' in key:
+        skyfile = os.path.join(conf_path, f'{key}_skyflat.fits')
+    elif key.startswith('nis-'):
+        skyfile = os.path.join(conf_path, f'{key}_skyflat.fits')
+    elif key.startswith('mirimage-'):
+        key += '-'+header['readpatt'].lower()
+        skyfile = os.path.join(conf_path, f'{key}_skyflat.fits')
+    else:
+        skyfile = os.path.join(conf_path, f'{key}_skyflat_smooth.fits')
+
+    if not os.path.exists(skyfile):
+        msg = f'jwst_utils.get_jwst_skyflat: {skyfile} not found'
+        utils.log_comment(utils.LOGFILE, msg, verbose=True)
+        return None, None, None
+
+    with pyfits.open(skyfile) as _im:
+        skyflat = _im[0].data*1
+        
+    if 'R_FLAT' in header:
+        oflat = os.path.basename(header['R_FLAT'])
+        crds_path = os.getenv('CRDS_PATH')
+        crds_path = os.path.join(crds_path, 'references/jwst', 
+                                 header['instrume'].lower(), oflat)
+        
+        msg = f'jwst_utils.get_jwst_skyflat: pipeline flat = {crds_path}\n'
+        
+        with pyfits.open(crds_path) as oim:
+            try:
+                flat_corr = oim['SCI'].data / skyflat            
+            except ValueError:
+                msg = f'jwst_utils.get_jwst_skyflat: flat_corr failed'
+                utils.log_comment(utils.LOGFILE, msg, verbose=True)
+                return None, None, None
+    else:
+        msg = f'jwst_utils.get_jwst_skyflat: NO pipeline flat\n'
+        flat_corr = 1./skyflat
+        
+    bad = skyflat < valid_flat[0]
+    bad |= skyflat > valid_flat[1]
+    bad |= ~np.isfinite(flat_corr)
+    flat_corr[bad] = 1
+    
+    dq = bad*1024
+
+    msg += f'jwst_utils.get_jwst_skyflat: new sky flat = {skyfile}\n'
+    msg += f'jwst_utils.get_jwst_skyflat: valid_flat={valid_flat}'
+    msg += f' nmask={bad.sum()}'
+    
+    if 'SUBSTRT1' in header:
+        if header['SUBSIZE1'] != 2048:
+            slx = slice(header['SUBSTRT1']-1,
+                        header['SUBSTRT1']-1 + header['SUBSIZE1'])
+            sly = slice(header['SUBSTRT2']-1,
+                        header['SUBSTRT2']-1 + header['SUBSIZE2'])
+            
+            msg += f"\njwst_utils.get_jwst_skyflat: subarray "
+            msg +=  header['APERNAME']
+            msg += f' [{sly.start}:{sly.stop},{slx.start}:{slx.stop}]'
+            
+            flat_corr = flat_corr[sly, slx]
+            dq = dq[sly, slx]
+            
+    utils.log_comment(utils.LOGFILE, msg, verbose=True)
+
+    return skyfile, flat_corr, dq
+
+
+def img_with_flat(input, verbose=True, overwrite=True, apply_photom=True, use_skyflats=True):
     """
     Apply flat-field and photom corrections if nessary
     """
@@ -239,6 +344,9 @@ def img_with_flat(input, verbose=True, overwrite=True, apply_photom=True):
                               verbose=verbose, show_date=False)
         
     else:
+        utils.log_comment(utils.LOGFILE,
+                          f'jwst_utils.img_with_flat: Flat already applied', 
+                          verbose=verbose, show_date=False)
         
         output = img
     
@@ -260,7 +368,29 @@ def img_with_flat(input, verbose=True, overwrite=True, apply_photom=True):
                                         'Applied photom')
                 
                 _hdu.flush()
-                
+        
+        if use_skyflats:
+            with pyfits.open(input, mode='update') as _hdu:
+                if 'FIXFLAT' not in _hdu[0].header:
+                    _sky = get_jwst_skyflat(_hdu[0].header)
+                    if _sky[0] is not None:
+                        if _hdu['SCI'].data.shape == _sky[1].shape:
+                            
+                            _hdu['SCI'].data *= _sky[1]
+
+                            _skyf = os.path.basename(_sky[0])
+                            _hdu[0].header['FIXFLAT'] = (True,
+                                               'Skyflat correction applied')
+                            _hdu[0].header['FIXFLATF'] = _skyf, 'Skyflat file'
+                            _dt = _hdu['DQ'].data.dtype
+                            _hdu['DQ'].data |= _sky[2].astype(_dt)
+                        
+                            _hdu.flush()
+                else:
+                    msg = f'jwst_utils.get_jwst_skyflat: FIXFLAT found'
+                    utils.log_comment(utils.LOGFILE, msg, 
+                                      verbose=verbose, show_date=False)
+                    
     gc.collect()
     
     return output
@@ -378,7 +508,8 @@ def img_with_wcs(input, overwrite=True, fit_sip_header=True, skip_completed=True
                                       'Distortion reference file')
                                       
         _hdu.writeto(input, overwrite=True)
-
+        _hdu.close()
+        
     return output
 
 
@@ -650,7 +781,220 @@ def copy_jwst_keywords(header, orig_keys=ORIG_KEYS, verbose=True):
                 utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
 
 
-def initialize_jwst_image(filename, verbose=True, max_dq_bit=14, orig_keys=ORIG_KEYS):
+def exposure_oneoverf_correction(file, axis=None, thresholds=[5,4,3], erode_mask=None, dilate_iterations=3, deg_pix=64, make_plot=True, init_model=0, in_place=False, skip_miri=True, verbose=True, **kwargs):
+    """
+    1/f correction for individual exposure
+    
+    1. Create a "background" mask with `sep`
+    2. Identify sources above threshold limit in the background-subtracted
+       image
+    3. Iterate a row/column correction on threshold-masked images.  A 
+       chebyshev polynomial is fit to the correction array to try to isolate
+       just the high-frequency oscillations.
+    
+    Parameters
+    ----------
+    file : str
+        JWST raw image filename
+    
+    axis : int
+        Axis over which to calculated the correction. If `None`, then defaults 
+        to ``axis=1`` (rows) for NIRCam and ``axis=1`` (columns) for NIRISS.
+    
+    thresholds : list
+        List of source identification thresholds
+        
+    erode_mask : bool
+        Erode the source mask to try to remove individual pixels that satisfy
+        the S/N threshold.  If `None`, then set to False if the exposure is a 
+        NIRISS dispersed image to avoid clipping compact high-order spectra
+        from the mask and True otherwise (for NIRISS imaging and NIRCam
+        generally).
+    
+    dilate_iterations : int
+        Number of `binary_dilation` iterations of the source mask
+        
+    deg_pix : int
+        Scale in pixels for each degree of the smooth chebyshev polynomial
+    
+    make_plot : bool
+        Make a diagnostic plot
+    
+    init_model : scalar, array-like
+        Initial correction model, e.g., for doing both axes
+    
+    in_place : bool
+        If True, remove the model from the 'SCI' extension of ``file``
+    
+    skip_miri : bool
+        Don't run on MIRI exposures
+    
+    verbose : bool
+        Print status messages
+        
+    Returns
+    -------
+    fig : `~matplotlib.figure.Figure`, None
+        Diagnostic figure if `make_plot=True`
+    
+    model : array-like
+        The row- or column-average correction array
+    """
+    import numpy as np
+    import scipy.ndimage as nd
+    import matplotlib.pyplot as plt
+    
+    import astropy.io.fits as pyfits
+    import sep
+    
+    im = pyfits.open(file)
+    if (im[0].header['INSTRUME'] in 'MIRI') & (skip_miri):
+        im.close()
+        
+        msg = 'exposure_oneoverf_correction: Skip for MIRI'
+        utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
+        
+        return None, 0
+        
+    if axis is None:
+        if im[0].header['INSTRUME'] in ('NIRISS', 'NIRSPEC'):
+            axis = 0
+        else:
+            axis = 1
+
+    msg = f'exposure_oneoverf_correction: {file} axis={axis} deg_pix={deg_pix}'
+    utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
+    
+    if im[0].header['INSTRUME'] in ('NIRSPEC'):
+        erode_mask = False
+        
+    if erode_mask is None:
+        if im[0].header['FILTER'].startswith('GR150'):
+            erode_mask = False
+        elif im[0].header['PUPIL'].startswith('GRISM'):
+            erode_mask = False
+        else:
+            erode_mask = True
+            
+    dq = utils.unset_dq_bits(im['DQ'].data, 4)
+    dqmask = dq == 0
+    mask = dqmask
+
+    err = im['ERR'].data
+    dqmask &= (err > 0) & np.isfinite(err)
+
+    sci = im['SCI'].data.astype(np.float32) - init_model
+    if deg_pix == 0:
+        bw = sci.shape[0] // 64
+    else:
+        bw = deg_pix
+        
+    bkg = sep.Background(sci, mask=~dqmask, bw=bw, bh=bw)
+    back = bkg.back()
+
+    sn_mask = (sci - back)/err > thresholds[0]
+    if erode_mask:
+        sn_mask = nd.binary_erosion(sn_mask)
+        
+    sn_mask = nd.binary_dilation(sn_mask, iterations=dilate_iterations)
+
+    mask = dqmask & ~sn_mask
+
+    cheb = 0
+    
+    if make_plot:
+        fig, ax = plt.subplots(1,1,figsize=(6,3))
+    else:
+        fig = None
+        
+    for _iter, thresh in enumerate(thresholds):
+        
+        if deg_pix == 0:
+            sci = im['SCI'].data*1. - init_model
+        else:
+            sci = im['SCI'].data*1. - back - init_model
+            
+        sci[~mask] = np.nan
+        med = np.nanmedian(sci, axis=axis)
+
+        if axis == 0:
+            model = np.zeros_like(sci) + (med - cheb)
+        else:
+            model = (np.zeros_like(sci) + (med - cheb)).T
+
+        sn_mask = ((sci-model)/err > thresh) & dqmask
+        if erode_mask:
+            sn_mask = nd.binary_erosion(sn_mask)
+        
+        sn_mask = nd.binary_dilation(sn_mask, iterations=dilate_iterations)
+        mask = dqmask & ~sn_mask
+        mask &= (sci-model)/err > -thresh
+
+        if make_plot:
+            ax.plot(med, alpha=0.5)
+
+    nx = med.size
+    xarr = np.linspace(-1,1,nx)
+    ok = np.isfinite(med)
+    
+    if deg_pix == 0:
+        # Don't remove anything from the median profile
+        cheb = 0.
+        deg = -1
+        
+    elif deg_pix >= nx:
+        # Remove constant component
+        cheb = np.nanmedian(med)
+        deg = 0
+        
+    else:
+        # Remove smooth component
+        deg = nx//deg_pix
+        
+        for _iter in range(3):
+            coeffs = np.polynomial.chebyshev.chebfit(xarr[ok], med[ok], deg)
+            cheb = np.polynomial.chebyshev.chebval(xarr, coeffs)
+            ok = np.isfinite(med) & (np.abs(med-cheb) < 0.05)
+
+        if make_plot:
+            ax.plot(np.arange(nx)[ok], cheb[ok], color='r')
+
+    if axis == 0:
+        model = np.zeros_like(sci) + (med - cheb)
+    else:
+        model = (np.zeros_like(sci) + (med - cheb)).T
+    
+    if make_plot:
+        ax.set_title(f'{file} axis={axis}')
+        ax.grid()
+        
+        fig.tight_layout(pad=0)
+    
+    im.close()
+    
+    if in_place: 
+        msg = f'exposure_oneoverf_correction: {file} apply to file'
+        utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
+        
+        with pyfits.open(file, mode='update') as im:
+            im[0].header['ONEFEXP'] = True, 'Exposure 1/f correction applied'
+            im[0].header['ONEFAXIS'] = axis, 'Axis for 1/f correction'
+            im[0].header['ONEFDEG'] = deg, 'Degree of smooth component'
+            im[0].header['ONEFNPIX'] = deg_pix, 'Pixels per smooth degree'
+        
+            model[~np.isfinite(model)] = 0
+        
+            im['SCI'].data -= model
+            im.flush()
+
+        if make_plot:
+            fig.savefig(file.split('.fits')[0] + f'_onef_axis{axis}.png')
+            plt.close('all')
+            
+    return fig, model
+
+
+def initialize_jwst_image(filename, verbose=True, max_dq_bit=14, orig_keys=ORIG_KEYS, oneoverf_correction=True, oneoverf_kwargs={'make_plot':False}, use_skyflats=True):
     """
     Make copies of some header keywords to make the headers look like 
     and HST instrument
@@ -658,8 +1002,9 @@ def initialize_jwst_image(filename, verbose=True, max_dq_bit=14, orig_keys=ORIG_
     1) Apply gain correction [x remove]
     2) Clip DQ bits
     3) Copy header keywords
-    4) Apply flat field if necessary
-    5) Initalize WCS
+    4) Apply exposure-level 1/f correction
+    5) Apply flat field if necessary
+    6) Initalize WCS
     
     Returns
     -------
@@ -779,7 +1124,7 @@ def initialize_jwst_image(filename, verbose=True, max_dq_bit=14, orig_keys=ORIG_
         # dq[:,:302] |= 1024
         
         # Dilate MIRI mask
-        msg = f'Dilate MIRI window mask'
+        msg = f'initialize_jwst_image: Dilate MIRI window mask'
         utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
         edge = nd.binary_dilation(((dq & 2**9) > 0), iterations=6)
         dq[edge] |= 1024
@@ -789,14 +1134,25 @@ def initialize_jwst_image(filename, verbose=True, max_dq_bit=14, orig_keys=ORIG_
         bpfile = os.path.join(os.path.dirname(__file__), 
                    f'data/nrc_lowpix_0916_{_det}.fits.gz')
         
-        if os.path.exists(bpfile):
+        if os.path.exists(bpfile) & False:
             bpdata = pyfits.open(bpfile)[0].data
-            bpdata = nd.binary_dilation(bpdata > 0, iterations=2)*1024
-            dq |= bpdata.astype(dq.dtype)
+            #bpdata = nd.binary_dilation(bpdata > 0, iterations=2)*1024
+            bpdata = nd.binary_dilation(bpdata > 0)*1024
+            if dq.shape == bpdata.shape:
+                dq |= bpdata.astype(dq.dtype)
             
-            msg = f'Use extra badpix in {bpfile}'
-            utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
+                msg = f'initialize_jwst_image: Use extra badpix in {bpfile}'
+                utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
         
+        if _det in ['NRCALONG','NRCBLONG']:
+            msg = 'initialize_jwst_image: Mask outer ring of 6 pixels'
+            msg += f' for {_det}'
+            utils.log_comment(utils.LOGFILE, msg, verbose=verbose)
+            dq[:6,:] |= 1024
+            dq[-6:,:] |= 1024
+            dq[:,:6] |= 1024
+            dq[:,-6:] |= 1024
+             
     img['DQ'].data = dq
     
     img[0].header['EXPTIME'] = img[0].header['EFFEXPTM']*1
@@ -817,7 +1173,59 @@ def initialize_jwst_image(filename, verbose=True, max_dq_bit=14, orig_keys=ORIG_
     img.writeto(filename, overwrite=True)
     img.close()
     
-    _ = img_with_flat(filename, overwrite=True)
+    _nircam_grism = False
+    
+    if oneoverf_correction:
+        
+        # NIRCam grism
+        if (img[0].header['OINSTRUM'] == 'NIRCAM'):
+             if 'GRISM' in img[0].header['OPUPIL']:
+                _nircam_grism = True
+
+        if not _nircam_grism:
+            
+            try:
+                _ = exposure_oneoverf_correction(filename, in_place=True, 
+                                             **oneoverf_kwargs)
+            except TypeError:
+                # Should only fail for test data
+                utils.log_exception(utils.LOGFILE, traceback)
+                msg = f'exposure_oneoverf_correction: failed for {filename}'
+                utils.log_comment(utils.LOGFILE, msg)
+            
+                pass
+                                         
+        # axis=None, thresholds=[5,4,3], erode_mask=None, dilate_iterations=3, deg_pix=64, make_plot=True, init_model=0, in_place=False, skip_miri=True, verbose=True
+        
+    _ = img_with_flat(filename, overwrite=True, use_skyflats=use_skyflats)
+    
+    # Now do "1/f" correction to subtract NIRCam grism sky
+    if _nircam_grism:
+        if img[0].header['OPUPIL'] == 'GRISMR':
+            _disp_axis = 0
+        else:
+            _disp_axis = 1
+        
+        msg = f'exposure_oneoverf_correction: NIRCam grism sky {filename}'
+        utils.log_comment(utils.LOGFILE, msg)
+
+        # # Subtract simple median
+        # exposure_oneoverf_correction(filename,
+        #                              in_place=True,
+        #                              erode_mask=False,
+        #                              thresholds=[4,3],
+        #                              axis=_disp_axis,
+        #                              dilate_iterations=5,
+        #                              deg_pix=0)
+
+        # Subtract average along dispersion axis    
+        exposure_oneoverf_correction(filename,
+                                     in_place=True, 
+                                     erode_mask=False,
+                                     thresholds=[4,3],
+                                     axis=_disp_axis,
+                                     dilate_iterations=5,
+                                     deg_pix=0)
 
     _ = img_with_wcs(filename, overwrite=True)
     
